==> wiki:public:forhunt_1.0.tar.gz: ForHunt/R/weed.proteins.R <==
weed.proteins <-
function(depVar, indepVar, pigengeneFile, outData, verbose=1, covars,
                          outPath=file.path(dirname(pigengeneFile), "weeding")){
    ## Using a greedy algorithm, we remove proteins one by one, recompute the eigenprotein,
    ##^ fit a linear model to this new eigenprotein, and measure the performance
    ## (based on p-value, R-square, etc.)
    ## This is useful to identify the "core" of a module, i.e., the proteins
    ## with the highest impact.
    ## Data:A data frame with participants on rows. Covariates and  outcomes on columns.
    ## Row names must be unique IDs for participants.
    ## depVar: A string showing the column in Data that is the dependent variable
    ## (e.g., "total_brain_pct").
    ## indevpVar: A string showing the column in Data that is the independent variable
    ## (e.g., "ME1").
    ## Output: The summary of the models,  R-square, etc.
    result <- list()
    rsqs <- c()
    pvals <- c()
    models <- list()
    coefs <- list()
    load(pigengeneFile, v=verbose>0) ## pigengene
    dir.create(outPath, showWarnings=FALSE, recursive=FALSE)
    absWeights <- sort(abs(pigengene$membership[,indepVar]))
    
    ## proteins in the module
    proteins <- names(which(pigengene$orderedModules==gsub(indepVar, pattern="ME", replacement="")))
    proteins <- names(sort(absWeights[proteins]))
    included <- proteins

    if(verbose>0){
        print(paste("Number of proteins:", length(absWeights)))
    }
    for(p1 in proteins){
        if(verbose > 2)
            print(p1)
        projected1 <- project.eigen(Data=pigengene$Data[ , included], pigengene=pigengene,
                                    saveFile=NULL, verbose=verbose-1)
        projected1 <- projected1$projected
       
        ## merge mri and protein data
        tot <- merge(projected1, outData, by=0)
        ## Remove participants with missing outcome variable.
        totsmall <- tot[!is.na(tot[[depVar]]), ]
        if(length(included) == length(proteins)) { ## Only in the first iteration,
            if(verbose>0){
                result[["sampleNum"]] <- nrow(tot)  ## Total number of participants
                ## remove people with missing depVar
                print(paste("Number of participants in", depVar, "analysis"))
                print(nrow(totsmall))
                result[["nonNaNum"]] <- nrow(totsmall)
            }
        }
        ## Fit a linear model:
        totsmall[indepVar] <- scale(totsmall[indepVar])
        model1 <- lin.model(depVar=depVar, indepVar=indepVar, covars=covars, Data=totsmall)
        models[[p1]] <- model1 ## The model with p1 and all proteins with a better weight than p1
        rsqs[p1] <- model1$rsq
        coefs[[p1]] <- model1$coefs

        pvals[p1] <- model1$coefs[indepVar,"Pr(>|t|)"]
        if(verbose>5)
            print(pvals[p1])
        if(pvals[p1]==min(pvals))
            minEigen <- projected1[,indepVar]
        ## Remove 1 protein:
        included <- names(sort(absWeights[included])[2:length(included)])
        if(length(included) <2)
            break
    }
    ## Plots:
    png(file.path(outPath, paste0(depVar,"pvals.png")))
    plot(y=pvals*4, x=rev(2:(length(pvals)+1)),
         ylab="Adjusted p-value", xlab="Number of proteins",
         xlim=rev(range(2:(length(pvals)+1))))
    abline(h=0.05)
    dev.off()
    png(file.path(outPath, "rsqs.png"))
    plot(y=rsqs, x=rev(2:(length(rsqs)+1)), ylab="R-square", xlab="Number of proteins")
    dev.off()

    result[["minEigens"]] <-minEigen 
    result[["proteins"]] <- proteins
    result[["rsqs"]] <- rsqs
    result[["models"]] <- models
    result[["coefs"]] <- coefs
    result[["pvals"]] <- pvals
    weeded <- result
    save(weeded, file=file.path(outPath, "weeded.Rdata"))
    return(result)
}

==> wiki:public:forhunt_1.2.tar.gz: ForHunt/R/weed.proteins.R <==
weed.proteins <-
function(depVar, indepVar, pigengeneFile, outData, verbose=1, covars,
                          outPath=file.path(dirname(pigengeneFile), "weeding", cohort=cohort)){
    ## Using a greedy algorithm, we remove proteins one by one, recompute the eigenprotein,
    ##^ fit a linear model to this new eigenprotein, and measure the performance
    ## (based on p-value, R-square, etc.)
    ## This is useful to identify the "core" of a module, i.e., the proteins
    ## with the highest impact.
    ## outData:A data frame with participants on rows. Covariates and  outcomes on columns.
    ## Row names must be unique IDs for participants.
    ## depVar: A string showing the column in Data that is the dependent variable
    ## (e.g., "total_brain_pct").
    ## indevpVar: A string showing the column in Data that is the independent variable
    ## (e.g., "ME1").
    ## Output: The summary of the models,  R-square, etc.
    result <- list()
    rsqs <- c()
    pvals <- c()
    models <- list()
    coefs <- list()
    load(pigengeneFile, v=verbose>0) ## pigengene
    if(cohort=="CHS")
        pigengene <- pigengeneNdata
    dir.create(outPath, showWarnings=FALSE, recursive=FALSE)
    absWeights <- sort(abs(pigengene$membership[,indepVar]))
    
    ## proteins in the module
    proteins <- names(which(pigengene$orderedModules==gsub(indepVar, pattern="ME", replacement="")))
    proteins <- names(sort(absWeights[proteins]))
    included <- proteins

    if(verbose>0){
        print(paste("Number of proteins:", length(absWeights)))
    }
    for(p1 in proteins){
        if(verbose > 2)
            print(p1)
        if(cohort!="CHS")
            dat1 <- pigengene$Data[ , included]
        if(cohort=="CHS"){
            load(file.path(dataPath,"resids.RData")) ##resids
            dat1 <- resids[ , included]
            }
        projected1 <- project.eigen(Data=dat1, pigengene=pigengene,
                                    saveFile=NULL, verbose=verbose-1)
        projected1 <- projected1$projected
       
        ## merge mri and protein data
        tot <- merge(projected1, outData, by=0)
        ## Remove participants with missing outcome variable.
        totsmall <- tot[!is.na(tot[[depVar]]), ]
        if(length(included) == length(proteins)) { ## Only in the first iteration,
            if(verbose>0){
                result[["sampleNum"]] <- nrow(tot)  ## Total number of participants
                ## remove people with missing depVar
                print(paste("Number of participants in", depVar, "analysis"))
                print(nrow(totsmall))
                result[["nonNaNum"]] <- nrow(totsmall)
            }
        }
        ## Fit a linear model:
        totsmall[indepVar] <- scale(totsmall[indepVar])
        model1 <- lin.model(depVar=depVar, indepVar=indepVar, covars=covars, Data=totsmall)
        models[[p1]] <- model1 ## The model with p1 and all proteins with a better weight than p1
        rsqs[p1] <- model1$rsq
        coefs[[p1]] <- model1$coefs

        pvals[p1] <- model1$coefs[indepVar,"Pr(>|t|)"]
        if(verbose>5)
            print(pvals[p1])
        if(pvals[p1]==min(pvals))
            minEigen <- projected1[,indepVar]
        ## Remove 1 protein:
        included <- names(sort(absWeights[included])[2:length(included)])
        if(length(included) <2)
            break
    }
    ## Plots:
    png(file.path(outPath, paste0(depVar,"pvals.png")))
    plot(y=pvals*4, x=rev(2:(length(pvals)+1)),
         ylab="Adjusted p-value", xlab="Number of proteins",
         xlim=rev(range(2:(length(pvals)+1))))
    abline(h=0.05)
    dev.off()
    png(file.path(outPath, "rsqs.png"))
    plot(y=rsqs, x=rev(2:(length(rsqs)+1)), ylab="R-square", xlab="Number of proteins")
    dev.off()

    result[["minEigens"]] <-minEigen 
    result[["proteins"]] <- proteins
    result[["rsqs"]] <- rsqs
    result[["models"]] <- models
    result[["coefs"]] <- coefs
    result[["pvals"]] <- pvals
    weeded <- result
    save(weeded, file=file.path(outPath, "weeded.Rdata"))
    return(result)
}
